/*****************************************************************************
 *
 * Atmel Corporation
 *
 * ̧		: AVRDevice.cpp
 * ߲	: Dev-C++ 4.9.8.0 - http://bloodshed.net/dev/
 * 		: $Revision: 4017 $
 * t		: $Date: 2008N62 14:26:03 +0200 $
 * XV	: $Author: khole $
 *
 * xҰ	: avr@atmel.com
 *
 * ړI	: Win32
 *
 * pLq	: AVR911 - AVRJ݊
 *
 * 		: ޲ؗʂȂǂ̏܂޸׽BAVR Studio 4ŋ
 *			  ޲Lq̧ق炻Ұǂނ߂̊֐񋟂܂B
 *
 ****************************************************************************/

#include "AVRDevice.hpp"

/* ݽ׸ */
AVRDevice::AVRDevice( const string & _deviceName ) :
	deviceName( _deviceName )
{
	flashSize = 0x4000;
	eepromSize = 0xFF;
	hasFuseBits = false;
	hasExtendedFuseBits = false;
	signature0 =
	signature1 =
	signature2 = 0;
	pagesize = 128;
}

/* ޽׸ */
AVRDevice::~AVRDevice()
{
	/* ̺ނȂ */
}

/* AVR StudioXMĻقҰǂݍ */	
void AVRDevice::readParametersFromAVRStudio( vector<string> & searchpath )
{
	string path;
	string signature;
	string cache;

#ifndef NOREGISTRY
	/* Windowsڼ޽ ްްXMĻق܂ިڸ؂ʒut */
	try
	{
		path = Util.getRegistryValue( "SOFTWARE\\Atmel\\AVRTools\\", "AVRToolsPath" );
		path += "\\PartDescriptionFiles";
		searchpath.push_back( path );
	} catch( ErrorMsg * e )
	{
		delete e;
	}
#endif

	/* ٌ̧ */
	path.erase();
	int i;
	for( i = 0; i < searchpath.size(); i++ )
	{
		path = searchpath[i] + "\\" + deviceName + ".xml";
		if( Util.fileExists( path ) )
			break;
	}

	if( i == searchpath.size() )
		throw new ErrorMsg( "Device XML file not found in search path!" );

	/* Kvȏ̂߂̧ق */
	Util.log( "Parsing '" + path + "'...\r\n" );
	XMLFile f( path );						// XMLݒ

	flashSize = atoi( f.getValue( "AVRPART\\MEMORY\\PROG_FLASH" ).c_str() );
	eepromSize = atoi( f.getValue( "AVRPART\\MEMORY\\EEPROM" ).c_str() );

	cache += "<AVRPART><MEMORY><PROG_FLASH>";
	cache += f.getValue( "AVRPART\\MEMORY\\PROG_FLASH" );
	cache += "</PROG_FLASH><EEPROM>";
	cache += f.getValue( "AVRPART\\MEMORY\\EEPROM" );
	cache += "</EEPROM>";

	if( f.exists( "AVRPART\\MEMORY\\BOOT_CONFIG" ) )
	{
		pagesize = atoi( f.getValue( "AVRPART\\MEMORY\\BOOT_CONFIG\\PAGESIZE" ).c_str() );
		pagesize <<= 1;						// ޲Ăł߰ޗeʂ]

		cache += "<BOOT_CONFIG><PAGESIZE>";
		cache += f.getValue( "AVRPART\\MEMORY\\BOOT_CONFIG\\PAGESIZE" );
		cache += "</PAGESIZE></BOOT_CONFIG>";
	}

	cache += "</MEMORY>";

	if( f.exists( "AVRPART\\FUSE" ) )
	{
		hasFuseBits = true;

		cache += "<FUSE>";

		if( f.exists( "AVRPART\\FUSE\\EXTENDED" ) )
		{
			hasExtendedFuseBits = true;
			cache += "<EXTENDED></EXTENDED>";
		}

		cache += "</FUSE>";
	}

	signature = f.getValue( "AVRPART\\ADMIN\\SIGNATURE\\ADDR000" );
	signature.erase( 0, 1 );				// $菜
	signature0 = Util.convertHex( signature );

	signature = f.getValue( "AVRPART\\ADMIN\\SIGNATURE\\ADDR001" );
	signature.erase( 0, 1 );				// $菜
	signature1 = Util.convertHex( signature );

	signature = f.getValue( "AVRPART\\ADMIN\\SIGNATURE\\ADDR002" );
	signature.erase( 0, 1 );				// $菜
	signature2 = Util.convertHex( signature );

	cache += "<ADMIN><SIGNATURE><ADDR000>";
	cache += f.getValue( "AVRPART\\ADMIN\\SIGNATURE\\ADDR000" );
	cache += "</ADDR000><ADDR001>";
	cache += f.getValue( "AVRPART\\ADMIN\\SIGNATURE\\ADDR001" );
	cache += "</ADDR001><ADDR002>";
	cache += f.getValue( "AVRPART\\ADMIN\\SIGNATURE\\ADDR002" );
	cache += "</ADDR002></SIGNATURE></ADMIN></AVRPART>\r\n";

	/* pިڸ؂ɷꂽ̧قۑ */
	Util.log( "Saving cached XML parameters...\r\n" );
	Util.saveString( cache, searchpath[1] + "\\" + deviceName + ".xml" );
}


void AVRDevice::getSignature( long * sig0, long * sig1, long * sig2 )
{
	if( sig0 == NULL || sig1 == NULL || sig2 == NULL )
		throw new ErrorMsg( "Cannot copy signature bytes to NULL-pointers!" );

	*sig0 = signature0;
	*sig1 = signature1;
	*sig2 = signature2;
}

/* ̧ق̍Ō */
